==> 1.0.0.tar.gz: lib/src/lists/Arguments.dart <==
import '../ParserExceptions.dart';
import '../ParserType.dart';
import '../lexer/Lexer.dart';
import '../main/Expression.dart';
import '../core/Label.dart';
import 'ExpressionList.dart';

/*
<arguments> ::= '(' (<argumentList> ','?)? ')'

See https://dart.dev/guides/language/specifications/DartLangSpec-v2.2.pdf
Page 104-105
*/
class Arguments extends ParserType {
  ArgumentList argumentList;
  bool hasComma;

  Arguments(Lexer lexer) {
    checkType(lexer, TType.LEFT_PAREN);
    lexer.advance();

    // print ('Started Arguments');

    checkNotType(lexer, TType.COMMA);

    try {
      argumentList = ArgumentList(lexer);
    }
    on WrongTokenType {}

    if (isType(lexer, TType.COMMA)) {
      lexer.advance();
      hasComma = true;
    }
    else {
      hasComma = false;
    }

    checkType(lexer, TType.RIGHT_PAREN);
    lexer.advance();

    // print ('Finished Arguments');
  }

  String toString() {
    return '(' + (argumentList != null ? '$argumentList' : '') + (hasComma ? ',' : '') + ')';
  }

}

/*
<argumentList> ::= <namedArgument> (',' <namedArgument>)*
| <expressionList> (',' <namedArgument>)*
*/
class ArgumentList extends ParserType {
  int caseNum;
  List<NamedArgument> argumentList;
  ExpressionList expressionList;

  ArgumentList(Lexer lexer) {
    argumentList = [];

    int attempt, maxAttempts = 2;
    int snapShot = lexer.currentIndex;
    for (attempt = 0; attempt < maxAttempts; ++attempt) {
      try {
        switch (attempt) {
          case 0: _case0(lexer); break;
          case 1: _case1(lexer); break;
        }

        break;
      }
      on WrongTokenType {
        lexer.setPos(snapShot);
      }
    }

    if (attempt == maxAttempts)
      throw WrongTokenType('Tried all options in ArgumentList: ALL FAILED');


  }
  
  /* <namedArgument> (',' <namedArgument>)* */
  void _case0(Lexer lexer) {
    argumentList.add(NamedArgument(lexer));

    int snapShot;
    try {
      while (isType(lexer, TType.COMMA)) {
        snapShot = lexer.currentIndex;
        lexer.advance();
        argumentList.add(NamedArgument(lexer));
      }
    }
    on WrongTokenType {
      lexer.setPos(snapShot);
    }

    caseNum = 0;
  }
  
  /* <expressionList> (',' <namedArgument>)* */
  void _case1(Lexer lexer) {
    expressionList = ExpressionList(lexer);

    int snapShot;
    try {
      while (isType(lexer, TType.COMMA)) {
        snapShot = lexer.currentIndex;
        lexer.advance();
        argumentList.add(NamedArgument(lexer));
      }
    }
    on WrongTokenType {
      lexer.setPos(snapShot);
    }

    caseNum = 1;
  }

  String toString() {
    switch (caseNum) {
      case 0:
        String rtn = '';
        argumentList.forEach((a) => rtn += ', $a');
        return rtn.substring(2);
        break;
      case 1:
        String rtn = '$expressionList';
        argumentList.forEach((a) => rtn += ', $a');
        return rtn;
        break;
    }
    return '';
  }
  
}

/* <namedArgument> ::= <label> <expression> */
class NamedArgument extends ParserType {
  Label label;
  Expression expression;

  NamedArgument(Lexer lexer) {
    label = Label(lexer);
    expression = Expression(lexer);
  }

  String toString() {
    return '$label $expression';
  }

}

==> 1.1.1.tar.gz: lib/src/lists/Arguments.dart <==
import '../ParserExceptions.dart';
import '../ParserType.dart';
import '../lexer/Lexer.dart';
import '../main/Expression.dart';
import '../core/Label.dart';
import 'ExpressionList.dart';

/// <arguments> ::= '(' (<argumentList> ','?)? ')'
///
/// See https://dart.dev/guides/language/specifications/DartLangSpec-v2.2.pdf
/// Page 104-105
class Arguments extends ParserType {
  ArgumentList argumentList;
  bool hasComma;

  Arguments(Lexer lexer) {
    hasComma = false;

    checkType(lexer, TType.LEFT_PAREN);
    lexer.advance();

    // print ('Started Arguments');

    checkNotType(lexer, TType.COMMA);

    int snapShot = lexer.currentIndex;
    try {
      argumentList = ArgumentList(lexer);

      if (isType(lexer, TType.COMMA)) {
        lexer.advance();
        hasComma = true;
      }
    } on WrongTokenType {
      lexer.setPos(snapShot);
    }

    checkType(lexer, TType.RIGHT_PAREN);
    lexer.advance();

    // print ('Finished Arguments');
  }

  String toString() {
    return '(' +
        (argumentList != null ? '$argumentList' : '') +
        (hasComma ? ',' : '') +
        ')';
  }
}

/// <argumentList> ::= <namedArgument> (',' <namedArgument>)*
/// | <expressionList> (',' <namedArgument>)*
class ArgumentList extends ParserType {
  int caseNum;
  List<NamedArgument> argumentList;
  ExpressionList expressionList;

  ArgumentList(Lexer lexer) {
    argumentList = [];

    int attempt, maxAttempts = 2;
    int snapShot = lexer.currentIndex;
    for (attempt = 0; attempt < maxAttempts; ++attempt) {
      try {
        switch (attempt) {
          case 0:
            _case0(lexer);
            break;
          case 1:
            _case1(lexer);
            break;
        }

        break;
      } on WrongTokenType {
        lexer.setPos(snapShot);
      }
    }

    if (attempt == maxAttempts) {
      throw WrongTokenType('Tried all options in ArgumentList: ALL FAILED');
    }
  }

  /// <namedArgument> (',' <namedArgument>)*
  void _case0(Lexer lexer) {
    argumentList.add(NamedArgument(lexer));

    int snapShot;
    try {
      while (isType(lexer, TType.COMMA)) {
        snapShot = lexer.currentIndex;
        lexer.advance();
        argumentList.add(NamedArgument(lexer));
      }
    } on WrongTokenType {
      lexer.setPos(snapShot);
    }

    caseNum = 0;
  }

  /// <expressionList> (',' <namedArgument>)*
  void _case1(Lexer lexer) {
    expressionList = ExpressionList(lexer);

    int snapShot;
    try {
      while (isType(lexer, TType.COMMA)) {
        snapShot = lexer.currentIndex;
        lexer.advance();
        argumentList.add(NamedArgument(lexer));
      }
    } on WrongTokenType {
      lexer.setPos(snapShot);
    }

    caseNum = 1;
  }

  String toString() {
    switch (caseNum) {
      case 0:
        String rtn = '';
        argumentList.forEach((a) => rtn += ', $a');
        return rtn.substring(2);
        break;
      case 1:
        String rtn = '$expressionList';
        argumentList.forEach((a) => rtn += ', $a');
        return rtn;
        break;
    }
    return '';
  }
}

/// <namedArgument> ::= <label> <expression>
class NamedArgument extends ParserType {
  Label label;
  Expression expression;

  NamedArgument(Lexer lexer) {
    label = Label(lexer);
    expression = Expression(lexer);
  }

  String toString() {
    return '$label $expression';
  }
}
